"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertDefinedTypeNode = exports.isDefinedTypeNode = exports.DefinedTypeNode = void 0;
const utils_1 = require("../utils");
const TypeNode_1 = require("./TypeNode");
class DefinedTypeNode {
    constructor(metadata, type) {
        this.nodeClass = 'DefinedTypeNode';
        this.metadata = { ...metadata, name: (0, utils_1.mainCase)(metadata.name) };
        this.type = type;
    }
    static fromIdl(idl) {
        var _a, _b, _c;
        const name = (_a = idl.name) !== null && _a !== void 0 ? _a : '';
        const docs = (_b = idl.docs) !== null && _b !== void 0 ? _b : [];
        const idlType = (_c = idl.type) !== null && _c !== void 0 ? _c : { kind: 'struct', fields: [] };
        const type = (0, TypeNode_1.createTypeNodeFromIdl)({ name, ...idlType });
        (0, TypeNode_1.assertTypeStructOrEnumNode)(type);
        return new DefinedTypeNode({ name, idlName: name, docs, importFrom: 'generated', internal: false }, type);
    }
    accept(visitor) {
        return visitor.visitDefinedType(this);
    }
    get name() {
        return this.metadata.name;
    }
    get docs() {
        return this.metadata.docs;
    }
}
exports.DefinedTypeNode = DefinedTypeNode;
function isDefinedTypeNode(node) {
    return !!node && node.nodeClass === 'DefinedTypeNode';
}
exports.isDefinedTypeNode = isDefinedTypeNode;
function assertDefinedTypeNode(node) {
    var _a;
    if (!isDefinedTypeNode(node)) {
        throw new Error(`Expected DefinedTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertDefinedTypeNode = assertDefinedTypeNode;
//# sourceMappingURL=DefinedTypeNode.js.map