"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertRootNode = exports.isRootNode = exports.RootNode = void 0;
const utils_1 = require("../utils");
const ProgramNode_1 = require("./ProgramNode");
class RootNode {
    constructor(programs) {
        this.nodeClass = 'RootNode';
        this.programs = programs;
    }
    static fromIdls(idls) {
        const programs = idls.map((idl) => ProgramNode_1.ProgramNode.fromIdl(idl));
        return new RootNode(programs);
    }
    static fromProgramInputs(inputs) {
        const inputArray = Array.isArray(inputs) ? inputs : [inputs];
        const idlArray = inputArray.map((program) => typeof program === 'string' ? (0, utils_1.readJson)(program) : program);
        return RootNode.fromIdls(idlArray);
    }
    accept(visitor) {
        return visitor.visitRoot(this);
    }
    get allAccounts() {
        return this.programs.flatMap((program) => program.accounts);
    }
    get allInstructions() {
        return this.programs.flatMap((program) => program.instructions);
    }
    get allInstructionsWithSubs() {
        return this.programs.flatMap((program) => program.instructionsWithSubs);
    }
    get allDefinedTypes() {
        return this.programs.flatMap((program) => program.definedTypes);
    }
}
exports.RootNode = RootNode;
function isRootNode(node) {
    return !!node && node.nodeClass === 'RootNode';
}
exports.isRootNode = isRootNode;
function assertRootNode(node) {
    var _a;
    if (!isRootNode(node)) {
        throw new Error(`Expected RootNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertRootNode = assertRootNode;
//# sourceMappingURL=RootNode.js.map