"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeLeafNode = exports.isTypeLeafNode = exports.TypeLeafNode = exports.LEAF_TYPES = void 0;
exports.LEAF_TYPES = [
    'string',
    'publicKey',
    'bytes',
    'bool',
    'u8',
    'u16',
    'u32',
    'u64',
    'u128',
    'i8',
    'i16',
    'i32',
    'i64',
    'i128',
    'f32',
    'f64',
];
class TypeLeafNode {
    constructor(type) {
        this.nodeClass = 'TypeLeafNode';
        this.type = type;
    }
    static isValidType(type) {
        return exports.LEAF_TYPES.includes(type);
    }
    accept(visitor) {
        return visitor.visitTypeLeaf(this);
    }
    isSignedInteger() {
        return ['i8', 'i16', 'i32', 'i64', 'i128'].includes(this.type);
    }
    isUnsignedInteger() {
        return ['u8', 'u16', 'u32', 'u64', 'u128'].includes(this.type);
    }
    isInteger() {
        return this.isSignedInteger() || this.isUnsignedInteger();
    }
    isDecimal() {
        return ['f32', 'f64'].includes(this.type);
    }
    isNumber() {
        return this.isInteger() || this.isDecimal();
    }
}
exports.TypeLeafNode = TypeLeafNode;
function isTypeLeafNode(node) {
    return !!node && node.nodeClass === 'TypeLeafNode';
}
exports.isTypeLeafNode = isTypeLeafNode;
function assertTypeLeafNode(node) {
    var _a;
    if (!isTypeLeafNode(node)) {
        throw new Error(`Expected TypeLeafNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeLeafNode = assertTypeLeafNode;
//# sourceMappingURL=TypeLeafNode.js.map