"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertTypeStructOrEnumNode = exports.isTypeStructOrEnumNode = exports.assertTypeNode = exports.isTypeNode = exports.createTypeNodeFromIdl = void 0;
const TypeArrayNode_1 = require("./TypeArrayNode");
const TypeDefinedLinkNode_1 = require("./TypeDefinedLinkNode");
const TypeEnumNode_1 = require("./TypeEnumNode");
const TypeLeafNode_1 = require("./TypeLeafNode");
const TypeMapNode_1 = require("./TypeMapNode");
const TypeOptionNode_1 = require("./TypeOptionNode");
const TypeSetNode_1 = require("./TypeSetNode");
const TypeStructNode_1 = require("./TypeStructNode");
const TypeTupleNode_1 = require("./TypeTupleNode");
const TypeVecNode_1 = require("./TypeVecNode");
const TYPE_NODE_CLASSES = [
    'TypeArrayNode',
    'TypeDefinedLinkNode',
    'TypeEnumNode',
    'TypeLeafNode',
    'TypeLeafWrapperNode',
    'TypeMapNode',
    'TypeOptionNode',
    'TypeSetNode',
    'TypeStructNode',
    'TypeTupleNode',
    'TypeVecNode',
];
function isArrayOfSize(array, size) {
    return Array.isArray(array) && array.length === size;
}
const createTypeNodeFromIdl = (idlType) => {
    // Leaf.
    if (typeof idlType === 'string' && TypeLeafNode_1.TypeLeafNode.isValidType(idlType)) {
        return new TypeLeafNode_1.TypeLeafNode(idlType);
    }
    // Ensure eveything else is an object.
    if (typeof idlType !== 'object') {
        throw new Error(`TypeNode: Unsupported type ${JSON.stringify(idlType)}`);
    }
    // Array.
    if ('array' in idlType && isArrayOfSize(idlType.array, 2)) {
        return TypeArrayNode_1.TypeArrayNode.fromIdl(idlType);
    }
    // Defined link.
    if ('defined' in idlType && typeof idlType.defined === 'string') {
        return new TypeDefinedLinkNode_1.TypeDefinedLinkNode(idlType.defined);
    }
    // Enum.
    if ('kind' in idlType && idlType.kind === 'enum' && 'variants' in idlType) {
        return TypeEnumNode_1.TypeEnumNode.fromIdl(idlType);
    }
    // Map.
    if (('hashMap' in idlType && isArrayOfSize(idlType.hashMap, 2)) ||
        ('bTreeMap' in idlType && isArrayOfSize(idlType.bTreeMap, 2))) {
        return TypeMapNode_1.TypeMapNode.fromIdl(idlType);
    }
    // Option.
    if ('option' in idlType || 'coption' in idlType) {
        return TypeOptionNode_1.TypeOptionNode.fromIdl(idlType);
    }
    // Set.
    if ('hashSet' in idlType || 'bTreeSet' in idlType) {
        return TypeSetNode_1.TypeSetNode.fromIdl(idlType);
    }
    // Struct.
    if ('kind' in idlType && idlType.kind === 'struct') {
        return TypeStructNode_1.TypeStructNode.fromIdl(idlType);
    }
    // Tuple.
    if ('tuple' in idlType && Array.isArray(idlType.tuple)) {
        return TypeTupleNode_1.TypeTupleNode.fromIdl(idlType);
    }
    // Vec.
    if ('vec' in idlType) {
        return TypeVecNode_1.TypeVecNode.fromIdl(idlType);
    }
    // Throw an error for unsupported types.
    throw new Error(`TypeNode: Unsupported type ${JSON.stringify(idlType)}`);
};
exports.createTypeNodeFromIdl = createTypeNodeFromIdl;
function isTypeNode(node) {
    return !!node && TYPE_NODE_CLASSES.includes(node.nodeClass);
}
exports.isTypeNode = isTypeNode;
function assertTypeNode(node) {
    var _a;
    if (!isTypeNode(node)) {
        throw new Error(`Expected TypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeNode = assertTypeNode;
function isTypeStructOrEnumNode(node) {
    return (!!node &&
        (node.nodeClass === 'TypeStructNode' || node.nodeClass === 'TypeEnumNode'));
}
exports.isTypeStructOrEnumNode = isTypeStructOrEnumNode;
function assertTypeStructOrEnumNode(node) {
    var _a;
    if (!isTypeStructOrEnumNode(node)) {
        throw new Error(`Expected TypeStructNode | TypeEnumNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertTypeStructOrEnumNode = assertTypeStructOrEnumNode;
//# sourceMappingURL=TypeNode.js.map