"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetJavaScriptValidatorBagVisitor = void 0;
const utils_1 = require("../../utils");
const nodes = __importStar(require("../../nodes"));
const visitors_1 = require("../../visitors");
class GetJavaScriptValidatorBagVisitor extends visitors_1.GetDefaultValidatorBagVisitor {
    constructor() {
        super(...arguments);
        this.exportMap = new Map();
    }
    visitRoot(root) {
        return super.visitRoot(root);
    }
    visitProgram(program) {
        const bag = super.visitProgram(program);
        this.pushNode(program);
        const pascalCaseName = (0, utils_1.pascalCase)(program.name);
        bag.mergeWith([
            this.checkExportConflicts(program, {
                [`get${pascalCaseName}Program`]: 'function',
                [`get${pascalCaseName}ErrorFromCode`]: 'function',
                [`get${pascalCaseName}ErrorFromName`]: 'function',
            }),
        ]);
        this.popNode();
        return bag;
    }
    visitAccount(account) {
        const bag = super.visitAccount(account);
        this.pushNode(account);
        const exports = {
            [account.name]: 'type',
            [`${account.name}AccountData`]: 'type',
            [`${account.name}AccountArgs`]: 'type',
            [`fetch${account.name}`]: 'function',
            [`safeFetch${account.name}`]: 'function',
            [`deserialize${account.name}`]: 'function',
            [`get${account.name}AccountDataSerializer`]: 'function',
            [`get${account.name}Size`]: 'function',
        };
        if (account.metadata.seeds.length > 0) {
            exports[`find${account.name}Pda`] = 'function';
        }
        bag.mergeWith([this.checkExportConflicts(account, exports)]);
        const reservedAccountFields = new Set(['publicKey', 'header']);
        const invalidFields = account.type.fields
            .map((field) => field.name)
            .filter((name) => reservedAccountFields.has(name));
        if (invalidFields.length > 0) {
            const x = invalidFields.join(', ');
            const message = invalidFields.length === 1
                ? `Account field [${x}] is reserved. Please rename it.`
                : `Account fields [${x}] are reserved. Please rename them.`;
            bag.error(message, account, this.stack);
        }
        this.popNode();
        return bag;
    }
    visitInstruction(instruction) {
        const bag = super.visitInstruction(instruction);
        this.pushNode(instruction);
        bag.mergeWith([
            this.checkExportConflicts(instruction, {
                [(0, utils_1.camelCase)(instruction.name)]: 'function',
                [`${instruction.name}InstructionAccounts`]: 'type',
                [`${instruction.name}InstructionData`]: 'type',
                [`${instruction.name}InstructionArgs`]: 'type',
                [`get${instruction.name}InstructionDataSerializer`]: 'function',
            }),
        ]);
        this.popNode();
        return bag;
    }
    visitDefinedType(definedType) {
        const bag = super.visitDefinedType(definedType);
        this.pushNode(definedType);
        const isDataEnum = nodes.isTypeEnumNode(definedType.type) && definedType.type.isDataEnum();
        bag.mergeWith([
            this.checkExportConflicts(definedType, {
                [definedType.name]: 'type',
                [`${definedType.name}Args`]: 'type',
                [`fetch${definedType.name}`]: 'function',
                ...(isDataEnum
                    ? {
                        [(0, utils_1.camelCase)(definedType.name)]: 'function',
                        [`is${definedType.name}`]: 'function',
                    }
                    : {}),
            }),
        ]);
        this.popNode();
        return bag;
    }
    visitError(error) {
        var _a, _b;
        const bag = super.visitError(error);
        this.pushNode(error);
        const prefixedName = (0, utils_1.pascalCase)((_b = (_a = this.program) === null || _a === void 0 ? void 0 : _a.metadata.prefix) !== null && _b !== void 0 ? _b : '') +
            (0, utils_1.pascalCase)(error.metadata.name);
        bag.mergeWith([
            this.checkExportConflicts(error, {
                [`${prefixedName}Error`]: 'class',
            }),
        ]);
        this.popNode();
        return bag;
    }
    visitTypeLeafWrapper(typeLeafWrapper) {
        const bag = super.visitTypeLeafWrapper(typeLeafWrapper);
        this.pushNode(typeLeafWrapper);
        const { wrapper, leaf } = typeLeafWrapper;
        switch (wrapper.kind) {
            case 'DateTime':
                if (!leaf.isInteger()) {
                    bag.error(`DateTime wrapper can only be applied to ` +
                        `integer types. Got type [${leaf.type}].`, typeLeafWrapper, this.stack);
                }
                break;
            case 'Amount':
                if (!leaf.isUnsignedInteger()) {
                    bag.error(`Amount wrapper can only be applied to ` +
                        `unsigned integer types. Got type [${leaf.type}].`, typeLeafWrapper, this.stack);
                }
                break;
            default:
                break;
        }
        this.popNode();
        return bag;
    }
    checkExportConflicts(node, exports) {
        const bag = new visitors_1.ValidatorBag();
        const stack = this.stack.clone();
        const conflictingNodes = [];
        Object.entries(exports).forEach(([exportName, exportType]) => {
            // Checks for conflicts.
            const exportConflict = this.exportMap.get(exportName);
            if (!exportConflict) {
                this.exportMap.set(exportName, { node, stack, exportType });
                return;
            }
            // Avoids throwing many similar error for the same kind of conflict.
            const conflictingNode = exportConflict.node;
            if (conflictingNodes.includes(conflictingNode))
                return;
            conflictingNodes.push(conflictingNode);
            // Constructs the error message.
            let exportDetails = '';
            let conflictExportDetails = '';
            if (!this.isEponymousExport(node, exportName)) {
                exportDetails = `exports a "${exportName}" ${exportType} that `;
            }
            if (!this.isEponymousExport(conflictingNode, exportName)) {
                conflictExportDetails = `"${exportName}" ${exportConflict.exportType} exported by the `;
            }
            const message = `The ${this.getNodeTitle(node)} ${exportDetails}` +
                `conflicts with the ${conflictExportDetails}` +
                `${this.getNodeTitle(conflictingNode)}.\n` +
                `|> Conflicting stack: ${exportConflict.stack}.`;
            bag.error(message, node, stack);
        });
        return bag;
    }
    isEponymousExport(node, exportName) {
        return exportName === ('name' in node ? node.name : '');
    }
    getNodeTitle(node) {
        const name = 'name' in node ? node.name : '';
        const type = (0, utils_1.titleCase)(node.nodeClass.slice(0, -4)).toLowerCase();
        return `"${name}" ${type}`;
    }
}
exports.GetJavaScriptValidatorBagVisitor = GetJavaScriptValidatorBagVisitor;
//# sourceMappingURL=GetJavaScriptValidatorBagVisitor.js.map