"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseNodeOrNullVisitor = void 0;
const nodes = __importStar(require("../nodes"));
class BaseNodeOrNullVisitor {
    visitRoot(root) {
        return new nodes.RootNode(root.programs
            .map((program) => program.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertProgramNode)));
    }
    visitProgram(program) {
        return new nodes.ProgramNode(program.metadata, program.accounts
            .map((account) => account.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertAccountNode)), program.instructions
            .map((instruction) => instruction.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertInstructionNode)), program.definedTypes
            .map((type) => type.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertDefinedTypeNode)), program.errors
            .map((error) => error.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertErrorNode)));
    }
    visitAccount(account) {
        const accountType = account.type.accept(this);
        if (accountType === null)
            return null;
        nodes.assertTypeStructNode(accountType);
        return new nodes.AccountNode(account.metadata, accountType);
    }
    visitInstruction(instruction) {
        const args = instruction.args.accept(this);
        if (args === null)
            return null;
        nodes.assertTypeStructNode(args);
        return new nodes.InstructionNode(instruction.metadata, instruction.accounts, args, instruction.subInstructions
            .map((ix) => ix.accept(this))
            .filter(nodes.removeNullAndAssertNodeFilter(nodes.assertInstructionNode)));
    }
    visitDefinedType(definedType) {
        const type = definedType.type.accept(this);
        if (type === null)
            return null;
        nodes.assertTypeStructOrEnumNode(type);
        return new nodes.DefinedTypeNode(definedType.metadata, type);
    }
    visitError(error) {
        return error;
    }
    visitTypeArray(typeArray) {
        const type = typeArray.itemType.accept(this);
        if (type === null)
            return null;
        nodes.assertTypeNode(type);
        return new nodes.TypeArrayNode(type, typeArray.size);
    }
    visitTypeDefinedLink(typeDefinedLink) {
        return typeDefinedLink;
    }
    visitTypeEnum(typeEnum) {
        const variants = typeEnum.variants
            .map((variant) => {
            const newVariant = variant.accept(this);
            if (newVariant === null)
                return null;
            nodes.assertTypeEnumVariantNode(newVariant);
            return newVariant;
        })
            .filter((v) => v !== null);
        return new nodes.TypeEnumNode(typeEnum.name, variants);
    }
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant) {
        return typeEnumEmptyVariant;
    }
    visitTypeEnumStructVariant(typeEnumStructVariant) {
        const newStruct = typeEnumStructVariant.struct.accept(this);
        if (!newStruct)
            return null;
        nodes.assertTypeStructNode(newStruct);
        return new nodes.TypeEnumStructVariantNode(typeEnumStructVariant.name, newStruct);
    }
    visitTypeEnumTupleVariant(typeEnumTupleVariant) {
        const newTuple = typeEnumTupleVariant.tuple.accept(this);
        if (!newTuple)
            return null;
        nodes.assertTypeTupleNode(newTuple);
        return new nodes.TypeEnumTupleVariantNode(typeEnumTupleVariant.name, newTuple);
    }
    visitTypeLeaf(typeLeaf) {
        return typeLeaf;
    }
    visitTypeLeafWrapper(typeLeafWrapper) {
        const leaf = typeLeafWrapper.leaf.accept(this);
        if (leaf === null)
            return null;
        nodes.assertTypeLeafNode(leaf);
        return new nodes.TypeLeafWrapperNode(typeLeafWrapper.wrapper, leaf);
    }
    visitTypeMap(typeMap) {
        const keyType = typeMap.keyType.accept(this);
        const valueType = typeMap.valueType.accept(this);
        if (keyType === null || valueType === null)
            return null;
        nodes.assertTypeNode(keyType);
        nodes.assertTypeNode(valueType);
        return new nodes.TypeMapNode(typeMap.mapType, keyType, valueType);
    }
    visitTypeOption(typeOption) {
        const type = typeOption.type.accept(this);
        if (type === null)
            return null;
        nodes.assertTypeNode(type);
        return new nodes.TypeOptionNode(typeOption.optionType, type);
    }
    visitTypeSet(typeSet) {
        const type = typeSet.type.accept(this);
        if (type === null)
            return null;
        nodes.assertTypeNode(type);
        return new nodes.TypeSetNode(typeSet.setType, type);
    }
    visitTypeStruct(typeStruct) {
        const fields = typeStruct.fields
            .map((field) => {
            const newField = field.accept(this);
            if (newField === null)
                return null;
            nodes.assertTypeStructFieldNode(newField);
            return newField;
        })
            .filter((field) => field !== null);
        return new nodes.TypeStructNode(typeStruct.name, fields);
    }
    visitTypeStructField(typeStructField) {
        const newType = typeStructField.type.accept(this);
        if (newType === null)
            return null;
        nodes.assertTypeNode(newType);
        return new nodes.TypeStructFieldNode(typeStructField.metadata, newType);
    }
    visitTypeTuple(typeTuple) {
        const items = typeTuple.itemTypes
            .map((type) => {
            const newType = type.accept(this);
            if (newType === null)
                return null;
            nodes.assertTypeNode(newType);
            return newType;
        })
            .filter((type) => type !== null);
        return new nodes.TypeTupleNode(items);
    }
    visitTypeVec(typeVec) {
        const type = typeVec.itemType.accept(this);
        if (type === null)
            return null;
        nodes.assertTypeNode(type);
        return new nodes.TypeVecNode(type);
    }
}
exports.BaseNodeOrNullVisitor = BaseNodeOrNullVisitor;
//# sourceMappingURL=BaseNodeOrNullVisitor.js.map