"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseVoidVisitor = void 0;
class BaseVoidVisitor {
    visitRoot(root) {
        root.programs.forEach((program) => program.accept(this));
    }
    visitProgram(program) {
        program.accounts.forEach((account) => account.accept(this));
        program.instructions.forEach((instruction) => instruction.accept(this));
        program.definedTypes.forEach((type) => type.accept(this));
        program.errors.forEach((type) => type.accept(this));
    }
    visitAccount(account) {
        account.type.accept(this);
        account.variableSeeds.forEach((seed) => seed.type.accept(this));
    }
    visitInstruction(instruction) {
        instruction.args.accept(this);
        instruction.subInstructions.forEach((ix) => ix.accept(this));
    }
    visitDefinedType(definedType) {
        definedType.type.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitError(error) {
        //
    }
    visitTypeArray(typeArray) {
        typeArray.itemType.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeDefinedLink(typeDefinedLink) {
        //
    }
    visitTypeEnum(typeEnum) {
        typeEnum.variants.forEach((variant) => variant.accept(this));
    }
    visitTypeEnumEmptyVariant(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    typeEnumEmptyVariant) {
        //
    }
    visitTypeEnumStructVariant(typeEnumStructVariant) {
        typeEnumStructVariant.struct.accept(this);
    }
    visitTypeEnumTupleVariant(typeEnumTupleVariant) {
        typeEnumTupleVariant.tuple.accept(this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitTypeLeaf(typeLeaf) {
        //
    }
    visitTypeLeafWrapper(typeLeafWrapper) {
        typeLeafWrapper.leaf.accept(this);
    }
    visitTypeMap(typeMap) {
        typeMap.keyType.accept(this);
        typeMap.valueType.accept(this);
    }
    visitTypeOption(typeOption) {
        typeOption.type.accept(this);
    }
    visitTypeSet(typeSet) {
        typeSet.type.accept(this);
    }
    visitTypeStruct(typeStruct) {
        typeStruct.fields.forEach((field) => field.accept(this));
    }
    visitTypeStructField(typeStructField) {
        typeStructField.type.accept(this);
    }
    visitTypeTuple(typeTuple) {
        typeTuple.itemTypes.forEach((itemType) => itemType.accept(this));
    }
    visitTypeVec(typeVec) {
        typeVec.itemType.accept(this);
    }
}
exports.BaseVoidVisitor = BaseVoidVisitor;
//# sourceMappingURL=BaseVoidVisitor.js.map