"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnwrapDefinedTypesVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const BaseNodeVisitor_1 = require("../BaseNodeVisitor");
class UnwrapDefinedTypesVisitor extends BaseNodeVisitor_1.BaseNodeVisitor {
    constructor(typesToInline = '*') {
        super();
        this.availableDefinedTypes = new Map();
        this.typesToInline =
            typesToInline === '*' ? '*' : typesToInline.map(utils_1.mainCase);
    }
    visitRoot(root) {
        root.allDefinedTypes.forEach((definedType) => {
            this.availableDefinedTypes.set(definedType.name, definedType);
        });
        return super.visitRoot(root);
    }
    visitProgram(program) {
        return new nodes.ProgramNode(program.metadata, program.accounts
            .map((account) => account.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertAccountNode)), program.instructions
            .map((instruction) => instruction.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)), program.definedTypes
            .filter((definedType) => !this.shouldInline(definedType.name))
            .map((type) => type.accept(this))
            .filter(nodes.assertNodeFilter(nodes.assertDefinedTypeNode)), program.errors);
    }
    visitTypeDefinedLink(typeDefinedLink) {
        if (!this.shouldInline(typeDefinedLink.definedType)) {
            return typeDefinedLink;
        }
        const definedType = this.availableDefinedTypes.get(typeDefinedLink.definedType);
        if (definedType === undefined) {
            throw new Error(`Trying to inline missing defined type [${typeDefinedLink.definedType}]. ` +
                `Ensure this visitor starts from the root node to access all defined types.`);
        }
        return definedType.type.accept(this);
    }
    shouldInline(definedType) {
        return (this.typesToInline === '*' || this.typesToInline.includes(definedType));
    }
}
exports.UnwrapDefinedTypesVisitor = UnwrapDefinedTypesVisitor;
//# sourceMappingURL=UnwrapDefinedTypesVisitor.js.map