"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThrowValidatorItemsVisitor = void 0;
/* eslint-disable no-console */
const chalk_1 = __importDefault(require("chalk"));
const logs_1 = require("../../logs");
const BaseDelegateVisitor_1 = require("../BaseDelegateVisitor");
class ThrowValidatorItemsVisitor extends BaseDelegateVisitor_1.BaseDelegateVisitor {
    constructor(validator, throwLevel = 'error') {
        super(validator);
        this.throwLevel = throwLevel;
    }
    map(validatorBag) {
        const bag = validatorBag.orderByLevel();
        bag.log();
        const levelHistogram = bag.items.reduce((acc, item) => {
            var _a;
            acc[item.level] = ((_a = acc[item.level]) !== null && _a !== void 0 ? _a : 0) + 1;
            return acc;
        }, {});
        const maxLevel = Object.keys(levelHistogram)
            .map((level) => (0, logs_1.getLevelIndex)(level))
            .sort((a, b) => b - a)[0];
        if (maxLevel >= (0, logs_1.getLevelIndex)(this.throwLevel)) {
            const histogramString = Object.keys(levelHistogram)
                .map((level) => `${level}s: ${levelHistogram[level]}`)
                .join(', ');
            console.log(`${chalk_1.default.red(`Failed to validate the nodes.`)} ` +
                `${chalk_1.default.red.bold(`Found ${histogramString}.`)}\n`);
            process.exit(1);
        }
    }
}
exports.ThrowValidatorItemsVisitor = ThrowValidatorItemsVisitor;
//# sourceMappingURL=ThrowValidatorItemsVisitor.js.map