import type { IdlAccount } from '../idl';
import type { Visitable, Visitor } from '../visitors';
import type { Node } from './Node';
import { TypeNode } from './TypeNode';
import { TypeStructNode } from './TypeStructNode';
import type { InstructionNodeAccountDefaultsSeed } from './InstructionNode';
import { TypeStructFieldNode } from './TypeStructFieldNode';
export type AccountNodeMetadata = {
    readonly name: string;
    readonly idlName: string;
    readonly docs: string[];
    readonly internal: boolean;
    readonly size: number | null;
    readonly discriminator: AccountNodeDiscriminator | null;
    readonly seeds: AccountNodeSeed[];
};
export type AccountNodeDiscriminator = {
    kind: 'field';
    name: string;
} | {
    kind: 'size';
};
export type AccountNodeSeed = {
    kind: 'programId';
} | {
    kind: 'literal';
    value: string;
} | {
    kind: 'variable';
    name: string;
    description: string;
    type: TypeNode;
};
export declare class AccountNode implements Visitable {
    readonly nodeClass: "AccountNode";
    readonly metadata: AccountNodeMetadata;
    readonly type: TypeStructNode;
    constructor(metadata: AccountNodeMetadata, type: TypeStructNode);
    static fromIdl(idl: Partial<IdlAccount>): AccountNode;
    accept<T>(visitor: Visitor<T>): T;
    get name(): string;
    get docs(): string[];
    get discriminatorField(): TypeStructFieldNode | null;
    get variableSeeds(): Extract<AccountNodeSeed, {
        kind: 'variable';
    }>[];
    get hasVariableSeeds(): boolean;
    get instructionAccountDefaultSeeds(): Record<string, InstructionNodeAccountDefaultsSeed>;
}
export declare function isAccountNode(node: Node | null): node is AccountNode;
export declare function assertAccountNode(node: Node | null): asserts node is AccountNode;
