import type { IdlInstruction } from '../idl';
import type { Dependency, Visitable, Visitor } from '../visitors';
import type { Node } from './Node';
import { TypeStructNode } from './TypeStructNode';
export type InstructionNodeMetadata = {
    name: string;
    idlName: string;
    docs: string[];
    internal: boolean;
    bytesCreatedOnChain: InstructionNodeBytesCreatedOnChain;
};
export type InstructionNodeAccount = {
    name: string;
    isWritable: boolean;
    isSigner: boolean;
    isOptionalSigner: boolean;
    isOptional: boolean;
    description: string;
    defaultsTo: InstructionNodeAccountDefaults;
};
export type InstructionNodeAccountDefaults = {
    kind: 'publicKey';
    publicKey: string;
} | {
    kind: 'account';
    name: string;
} | {
    kind: 'identity';
} | {
    kind: 'payer';
} | {
    kind: 'pda';
    pdaAccount: string;
    dependency: Dependency;
    seeds: Record<string, InstructionNodeAccountDefaultsSeed>;
} | {
    kind: 'program';
    program: {
        name: string;
        publicKey: string;
    };
} | {
    kind: 'programId';
} | {
    kind: 'none';
};
export type InstructionNodeAccountDefaultsSeed = {
    kind: 'account';
    name: string;
} | {
    kind: 'arg';
    name: string;
};
export type InstructionNodeBytesCreatedOnChain = {
    kind: 'number';
    value: number;
    includeHeader: boolean;
} | {
    kind: 'arg';
    name: string;
    includeHeader: boolean;
} | {
    kind: 'account';
    name: string;
    dependency: string;
    includeHeader: boolean;
} | {
    kind: 'none';
};
export declare class InstructionNode implements Visitable {
    readonly nodeClass: "InstructionNode";
    readonly metadata: InstructionNodeMetadata;
    readonly accounts: InstructionNodeAccount[];
    readonly args: TypeStructNode;
    readonly subInstructions: InstructionNode[];
    constructor(metadata: InstructionNodeMetadata, accounts: InstructionNodeAccount[], args: TypeStructNode, subInstructions: InstructionNode[]);
    static fromIdl(idl: Partial<IdlInstruction>): InstructionNode;
    accept<T>(visitor: Visitor<T>): T;
    getAllSubInstructions(): InstructionNode[];
    get name(): string;
    get docs(): string[];
    get hasAccounts(): boolean;
    get hasData(): boolean;
    get hasArgs(): boolean;
    get hasRequiredArgs(): boolean;
}
export declare function isInstructionNode(node: Node | null): node is InstructionNode;
export declare function assertInstructionNode(node: Node | null): asserts node is InstructionNode;
