import * as nodes from '../../nodes';
import { Visitor } from '../../visitors';
import { JavaScriptImportMap } from './JavaScriptImportMap';
export type JavaScriptTypeManifest = {
    strictType: string;
    looseType: string;
    hasLooseType: boolean;
    isEnum: boolean;
    serializer: string;
    imports: JavaScriptImportMap;
};
export declare class GetJavaScriptTypeManifestVisitor implements Visitor<JavaScriptTypeManifest> {
    readonly serializerVariable: string;
    private availableDefinedTypes;
    private visitedDefinedTypes;
    private definedTypeStack;
    private definedName;
    constructor(serializerVariable?: string);
    registerDefinedTypes(definedTypes: nodes.DefinedTypeNode[]): void;
    visitRoot(): JavaScriptTypeManifest;
    visitProgram(): JavaScriptTypeManifest;
    visitAccount(account: nodes.AccountNode): JavaScriptTypeManifest;
    visitInstruction(instruction: nodes.InstructionNode): JavaScriptTypeManifest;
    visitDefinedType(definedType: nodes.DefinedTypeNode): JavaScriptTypeManifest;
    visitError(): JavaScriptTypeManifest;
    visitTypeArray(typeArray: nodes.TypeArrayNode): JavaScriptTypeManifest;
    visitTypeDefinedLink(typeDefinedLink: nodes.TypeDefinedLinkNode): JavaScriptTypeManifest;
    visitTypeEnum(typeEnum: nodes.TypeEnumNode): JavaScriptTypeManifest;
    visitTypeEnumEmptyVariant(typeEnumEmptyVariant: nodes.TypeEnumEmptyVariantNode): JavaScriptTypeManifest;
    visitTypeEnumStructVariant(typeEnumStructVariant: nodes.TypeEnumStructVariantNode): JavaScriptTypeManifest;
    visitTypeEnumTupleVariant(typeEnumTupleVariant: nodes.TypeEnumTupleVariantNode): JavaScriptTypeManifest;
    visitTypeLeaf(typeLeaf: nodes.TypeLeafNode): JavaScriptTypeManifest;
    visitTypeLeafWrapper(typeLeafWrapper: nodes.TypeLeafWrapperNode): JavaScriptTypeManifest;
    visitTypeMap(typeMap: nodes.TypeMapNode): JavaScriptTypeManifest;
    visitTypeOption(typeOption: nodes.TypeOptionNode): JavaScriptTypeManifest;
    visitTypeSet(typeSet: nodes.TypeSetNode): JavaScriptTypeManifest;
    visitTypeStruct(typeStruct: nodes.TypeStructNode): JavaScriptTypeManifest;
    visitTypeStructField(typeStructField: nodes.TypeStructFieldNode): JavaScriptTypeManifest;
    visitTypeTuple(typeTuple: nodes.TypeTupleNode): JavaScriptTypeManifest;
    visitTypeVec(typeVec: nodes.TypeVecNode): JavaScriptTypeManifest;
    protected s(name: string): string;
    protected mergeManifests(manifests: JavaScriptTypeManifest[]): Pick<JavaScriptTypeManifest, 'imports' | 'hasLooseType' | 'isEnum'>;
    protected createDocblock(docs: string[]): string;
}
